@@ -3,7 +3,7 @@
  *  This include file includes all other public GiNaC headers. */
 
 /*
- *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -76,9 +76,4 @@
 #include "parser/parser.h"
 #endif
 
-#ifdef __MAKECINT__
-#pragma link C++ nestedclass;
-#pragma link C++ nestedtypedef;
-#endif
-
 #endif // ndef GINAC_H
