@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's initially known functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "ex.h"
 #include "constant.h"
 #include "lst.h"
+#include "fderivative.h"
 #include "matrix.h"
 #include "mul.h"
 #include "power.h"
@@ -66,6 +67,19 @@ static ex conjugate_conjugate(const ex & arg)
 	return arg;
 }
 
+// If x is real then U.diff(x)-I*V.diff(x) represents both conjugate(U+I*V).diff(x) 
+// and conjugate((U+I*V).diff(x))
+static ex conjugate_expl_derivative(const ex & arg, const symbol & s)
+{
+	if (s.info(info_flags::real))
+		return conjugate(arg.diff(s));
+	else {
+		exvector vec_arg;
+		vec_arg.push_back(arg);
+		return fderivative(ex_to<function>(conjugate(arg)).get_serial(),0,vec_arg).hold()*arg.diff(s);
+	}
+}
+
 static ex conjugate_real_part(const ex & arg)
 {
 	return arg.real_part();
@@ -95,7 +109,6 @@ static bool func_arg_info(const ex & arg, unsigned inf)
 		case info_flags::prime:
 		case info_flags::crational_polynomial:
 		case info_flags::rational_function:
-		case info_flags::algebraic:
 		case info_flags::positive:
 		case info_flags::negative:
 		case info_flags::nonnegative:
@@ -115,6 +128,7 @@ static bool conjugate_info(const ex & arg, unsigned inf)
 
 REGISTER_FUNCTION(conjugate_function, eval_func(conjugate_eval).
                                       evalf_func(conjugate_evalf).
+                                      expl_derivative_func(conjugate_expl_derivative).
                                       info_func(conjugate_info).
                                       print_func<print_latex>(conjugate_print_latex).
                                       conjugate_func(conjugate_conjugate).
@@ -159,8 +173,21 @@ static ex real_part_imag_part(const ex & arg)
 	return 0;
 }
 
+// If x is real then Re(e).diff(x) is equal to Re(e.diff(x)) 
+static ex real_part_expl_derivative(const ex & arg, const symbol & s)
+{
+	if (s.info(info_flags::real))
+		return real_part_function(arg.diff(s));
+	else {
+		exvector vec_arg;
+		vec_arg.push_back(arg);
+		return fderivative(ex_to<function>(real_part(arg)).get_serial(),0,vec_arg).hold()*arg.diff(s);
+	}
+}
+
 REGISTER_FUNCTION(real_part_function, eval_func(real_part_eval).
                                       evalf_func(real_part_evalf).
+                                      expl_derivative_func(real_part_expl_derivative).
                                       print_func<print_latex>(real_part_print_latex).
                                       conjugate_func(real_part_conjugate).
                                       real_part_func(real_part_real_part).
@@ -204,8 +231,21 @@ static ex imag_part_imag_part(const ex & arg)
 	return 0;
 }
 
+// If x is real then Im(e).diff(x) is equal to Im(e.diff(x)) 
+static ex imag_part_expl_derivative(const ex & arg, const symbol & s)
+{
+	if (s.info(info_flags::real))
+		return imag_part_function(arg.diff(s));
+	else {
+		exvector vec_arg;
+		vec_arg.push_back(arg);
+		return fderivative(ex_to<function>(imag_part(arg)).get_serial(),0,vec_arg).hold()*arg.diff(s);
+	}
+}
+
 REGISTER_FUNCTION(imag_part_function, eval_func(imag_part_eval).
                                       evalf_func(imag_part_evalf).
+                                      expl_derivative_func(imag_part_expl_derivative).
                                       print_func<print_latex>(imag_part_print_latex).
                                       conjugate_func(imag_part_conjugate).
                                       real_part_func(imag_part_real_part).
@@ -232,6 +272,9 @@ static ex abs_eval(const ex & arg)
 	if (arg.info(info_flags::nonnegative))
 		return arg;
 
+	if (arg.info(info_flags::negative) || (-arg).info(info_flags::nonnegative))
+		return -arg;
+
 	if (is_ex_the_function(arg, abs))
 		return arg;
 
@@ -266,7 +309,7 @@ static ex abs_expand(const ex & arg, unsigned options)
 			else
 				prodseq.push_back(abs(*i));
 		}
-		return (new mul(prodseq))->setflag(status_flags::dynallocated | status_flags::expanded);
+		return dynallocate<mul>(prodseq).setflag(status_flags::expanded);
 	}
 
 	if (options & expand_options::expand_function_args)
@@ -275,6 +318,12 @@ static ex abs_expand(const ex & arg, unsigned options)
 		return abs(arg).hold();
 }
 
+static ex abs_expl_derivative(const ex & arg, const symbol & s)
+{
+	ex diff_arg = arg.diff(s);
+	return (diff_arg*arg.conjugate()+arg*diff_arg.conjugate())/2/abs(arg);
+}
+
 static void abs_print_latex(const ex & arg, const print_context & c)
 {
 	c.s << "{|"; arg.print(c); c.s << "|}";
@@ -302,10 +351,12 @@ static ex abs_imag_part(const ex& arg)
 
 static ex abs_power(const ex & arg, const ex & exp)
 {
-	if (arg.is_equal(arg.conjugate()) && ((is_a<numeric>(exp) && ex_to<numeric>(exp).is_even())
-						|| exp.info(info_flags::even)))
-		return power(arg, exp);
-	else
+	if ((is_a<numeric>(exp) && ex_to<numeric>(exp).is_even()) || exp.info(info_flags::even)) {
+		if (arg.info(info_flags::real) || arg.is_equal(arg.conjugate()))
+			return pow(arg, exp);
+		else
+			return pow(arg, exp/2) * pow(arg.conjugate(), exp/2);
+	} else
 		return power(abs(arg), exp).hold();
 }
 
@@ -339,6 +390,7 @@ bool abs_info(const ex & arg, unsigned inf)
 REGISTER_FUNCTION(abs, eval_func(abs_eval).
                        evalf_func(abs_evalf).
                        expand_func(abs_expand).
+                       expl_derivative_func(abs_expl_derivative).
                        info_func(abs_info).
                        print_func<print_latex>(abs_print_latex).
                        print_func<print_csrc_float>(abs_print_csrc_float).
@@ -400,9 +452,8 @@ static ex step_series(const ex & arg,
 	    && !(options & series_options::suppress_branchcut))
 		throw (std::domain_error("step_series(): on imaginary axis"));
 	
-	epvector seq;
-	seq.push_back(expair(step(arg_pt), _ex0));
-	return pseries(rel,seq);
+	epvector seq { expair(step(arg_pt), _ex0) };
+	return pseries(rel, std::move(seq));
 }
 
 static ex step_conjugate(const ex& arg)
@@ -479,9 +530,8 @@ static ex csgn_series(const ex & arg,
 	    && !(options & series_options::suppress_branchcut))
 		throw (std::domain_error("csgn_series(): on imaginary axis"));
 	
-	epvector seq;
-	seq.push_back(expair(csgn(arg_pt), _ex0));
-	return pseries(rel,seq);
+	epvector seq { expair(csgn(arg_pt), _ex0) };
+	return pseries(rel, std::move(seq));
 }
 
 static ex csgn_conjugate(const ex& arg)
@@ -587,9 +637,8 @@ static ex eta_series(const ex & x, const ex & y,
 	    (y_pt.info(info_flags::numeric) && y_pt.info(info_flags::negative)) ||
 	    ((x_pt*y_pt).info(info_flags::numeric) && (x_pt*y_pt).info(info_flags::negative)))
 			throw (std::domain_error("eta_series(): on discontinuity"));
-	epvector seq;
-	seq.push_back(expair(eta(x_pt,y_pt), _ex0));
-	return pseries(rel,seq);
+	epvector seq { expair(eta(x_pt,y_pt), _ex0) };
+	return pseries(rel, std::move(seq));
 }
 
 static ex eta_conjugate(const ex & x, const ex & y)
@@ -692,9 +741,8 @@ static ex Li2_series(const ex &x, const relational &rel, int order, unsigned opt
 			// substitute the argument's series expansion
 			ser = ser.subs(s==x.series(rel, order), subs_options::no_pattern);
 			// maybe that was terminating, so add a proper order term
-			epvector nseq;
-			nseq.push_back(expair(Order(_ex1), order));
-			ser += pseries(rel, nseq);
+			epvector nseq { expair(Order(_ex1), order) };
+			ser += pseries(rel, std::move(nseq));
 			// reexpanding it will collapse the series again
 			return ser.series(rel, order);
 			// NB: Of course, this still does not allow us to compute anything
@@ -717,9 +765,8 @@ static ex Li2_series(const ex &x, const relational &rel, int order, unsigned opt
 			// substitute the argument's series expansion
 			ser = ser.subs(s==x.series(rel, order), subs_options::no_pattern);
 			// maybe that was terminating, so add a proper order term
-			epvector nseq;
-			nseq.push_back(expair(Order(_ex1), order));
-			ser += pseries(rel, nseq);
+			epvector nseq { expair(Order(_ex1), order) };
+			ser += pseries(rel, std::move(nseq));
 			// reexpanding it will collapse the series again
 			return ser.series(rel, order);
 		}
@@ -741,7 +788,7 @@ static ex Li2_series(const ex &x, const relational &rel, int order, unsigned opt
 				seq.push_back(expair((replarg.op(i)/power(s-foo,i)).series(foo==point,1,options).op(0).subs(foo==s, subs_options::no_pattern),i));
 			// append an order term:
 			seq.push_back(expair(Order(_ex1), replarg.nops()-1));
-			return pseries(rel, seq);
+			return pseries(rel, std::move(seq));
 		}
 	}
 	// all other cases should be safe, by now:
@@ -903,7 +950,7 @@ static ex binomial_eval(const ex & x, const ex &y)
 		return binomial(x, y).hold();
 }
 
-// At the moment the numeric evaluation of a binomail function always
+// At the moment the numeric evaluation of a binomial function always
 // gives a real number, but if this would be implemented using the gamma
 // function, also complex conjugation should be changed (or rather, deleted).
 static ex binomial_conjugate(const ex & x, const ex & y)
@@ -951,11 +998,10 @@ static ex Order_eval(const ex & x)
 static ex Order_series(const ex & x, const relational & r, int order, unsigned options)
 {
 	// Just wrap the function into a pseries object
-	epvector new_seq;
 	GINAC_ASSERT(is_a<symbol>(r.lhs()));
 	const symbol &s = ex_to<symbol>(r.lhs());
-	new_seq.push_back(expair(Order(_ex1), numeric(std::min(x.ldegree(s), order))));
-	return pseries(r, new_seq);
+	epvector new_seq { expair(Order(_ex1), numeric(std::min(x.ldegree(s), order))) };
+	return pseries(r, std::move(new_seq));
 }
 
 static ex Order_conjugate(const ex & x)
@@ -975,11 +1021,15 @@ static ex Order_imag_part(const ex & x)
 	return Order(x).hold();
 }
 
-// Differentiation is handled in function::derivative because of its special requirements
+static ex Order_expl_derivative(const ex & arg, const symbol & s)
+{
+	return Order(arg.diff(s));
+}
 
 REGISTER_FUNCTION(Order, eval_func(Order_eval).
                          series_func(Order_series).
                          latex_name("\\mathcal{O}").
+                         expl_derivative_func(Order_expl_derivative).
                          conjugate_func(Order_conjugate).
                          real_part_func(Order_real_part).
                          imag_part_func(Order_imag_part));
@@ -994,7 +1044,7 @@ ex lsolve(const ex &eqns, const ex &symbols, unsigned options)
 	if (eqns.info(info_flags::relation_equal)) {
 		if (!symbols.info(info_flags::symbol))
 			throw(std::invalid_argument("lsolve(): 2nd argument must be a symbol"));
-		const ex sol = lsolve(lst(eqns),lst(symbols));
+		const ex sol = lsolve(lst{eqns}, lst{symbols});
 		
 		GINAC_ASSERT(sol.nops()==1);
 		GINAC_ASSERT(is_exactly_a<relational>(sol.op(0)));
@@ -1052,12 +1102,12 @@ ex lsolve(const ex &eqns, const ex &symbols, unsigned options)
 	} catch (const std::runtime_error & e) {
 		// Probably singular matrix or otherwise overdetermined system:
 		// It is consistent to return an empty list
-		return lst();
+		return lst{};
 	}
 	GINAC_ASSERT(solution.cols()==1);
 	GINAC_ASSERT(solution.rows()==symbols.nops());
 	
-	// return list of equations of the form lst(var1==sol1,var2==sol2,...)
+	// return list of equations of the form lst{var1==sol1,var2==sol2,...}
 	lst sollist;
 	for (size_t i=0; i<symbols.nops(); i++)
 		sollist.append(symbols.op(i)==solution(i,0));
@@ -1167,7 +1217,7 @@ fsolve(const ex& f_in, const symbol& x, const numeric& x1, const numeric& x2)
 			// determined by the secant between the values xx[0] and xx[1].
 			// Don't set the secant_weight to one because that could disturb
 			// the convergence in some corner cases!
-			static const double secant_weight = 0.984375;  // == 63/64 < 1
+			constexpr double secant_weight = 0.984375;  // == 63/64 < 1
 			numeric xxmid = (1-secant_weight)*0.5*(xx[0]+xx[1])
 			    + secant_weight*(xx[0]+fx[0]*(xx[0]-xx[1])/(fx[1]-fx[0]));
 			ex fxmid_ = f.subs(x == xxmid).evalf();
